// Copyright 2011 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.api.ads.dfa.lib.auth;

import com.google.api.ads.dfa.lib.client.DfaServiceClient;
import com.google.api.ads.dfa.lib.client.DfaServiceSession;
import com.google.api.ads.dfa.lib.factory.DfaServiceClientFactory;
import com.google.common.annotations.VisibleForTesting;
import com.google.inject.Inject;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * A utility class that can authenticate against the DFA API and return
 * authentication tokens.
 *
 * @author api.jdilallo@gmail.com (Joseph DiLallo)
 */
public class LoginTicketProvider {

  private final DfaServiceClientFactory dfaServiceClientFactory;

  /**
   * Constructor.
   */
  @Inject
  public LoginTicketProvider(DfaServiceClientFactory dfaServiceClientFactory) {
    this.dfaServiceClientFactory = dfaServiceClientFactory;
  }

  /**
   * Generates a new authentication token using reflection.
   *
   * @param stubPackage the package where all generated stubs are located
   * @param session the DFA service session
   * @return an authentication token
   * @throws LoginTicketException thrown if generating a token fails
   */
  public String generateToken(String stubPackage, DfaServiceSession session)
      throws LoginTicketException {
    try {
      // Instantiate a SOAP client connected to the login service.
      Class<?> loginClass = getLoginClass(stubPackage);
      Object loginService = getLoginService(session, loginClass);

      // Authenticate.
      Object userProfile = authenticate(session, loginClass, loginService);

      // Fetch the token from the returned UserProfile.
      return retrieveToken(userProfile);
    } catch (ClassNotFoundException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    } catch (NoSuchMethodException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    } catch (InvocationTargetException e) {
      throw new LoginTicketException("Unable to generate authentication token because of error:",
          e.getTargetException());
    } catch (IllegalAccessException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    }
  }

  /**
   * Retrieves the {@code Class} object representing the login service.
   *
   * @param stubPackage the package where all generated stubs are located
   * @return the {@code Class} object representing the login service
   * @throws ClassNotFoundException if the class cannot be resolved
   */
  @VisibleForTesting
  Class<?> getLoginClass(String stubPackage) throws ClassNotFoundException {
    return Class.forName(stubPackage + ".LoginRemote");
  }

  /**
   * Returns a {@link DfaServiceClient} wrapping the login service.
   *
   * @param session the DFA service session
   * @param loginClass the {@code Class} type of the login service
   * @return a client pointing to the login service
   */
  @VisibleForTesting
  Object getLoginService(DfaServiceSession session, Class<?> loginClass) {
    return dfaServiceClientFactory.getServiceClientAsInterface(session, loginClass);
  }

  /**
   * Authenticates against the DFA API.
   *
   * @param loginClass the {@code Class} type of the login service
   * @param loginService a service client object wrapping the login service
   * @return the {@code UserProfile} response from the API
   * @throws IllegalArgumentException if one of the arguments used in
   *     authenticating is not allowed
   * @throws IllegalAccessException if the {@code authenticate} method cannot be
   *     accessed
   * @throws InvocationTargetException if an error occurs while invoking the
   *     {@code authenticate} method
   * @throws NoSuchMethodException if the {@code authenticate method} cannot be
   *     located
   */
  @VisibleForTesting
  Object authenticate(DfaServiceSession session, Class<?> loginClass, Object loginService)
      throws IllegalArgumentException, IllegalAccessException, InvocationTargetException,
      NoSuchMethodException {
    Class<?> paramTypes[] = {String.class, String.class};
    Method authMethod = loginClass.getDeclaredMethod("authenticate", paramTypes);
    Object argList[] = {session.getAdsUser().getUsername(), session.getAdsUser().getPassword()};
    return authMethod.invoke(loginService, argList);
  }

  /**
   * Retrieves the {@code Class} object representing user profiles.
   *
   * @param stubPackage the package where all generated stubs are located
   * @return the {@code Class} object representing user profiles
   * @throws ClassNotFoundException if the class cannot be resolved
   */
  @VisibleForTesting
  Class<?> getUserProfileClass(String stubPackage) throws ClassNotFoundException {
    return Class.forName(stubPackage + ".UserProfile");
  }

  /**
   * Retrieves the user's authentication token from the {@code UserProfile}
   * returned by the DFA API.
   *
   * @param userProfile the {@code UserProfile} object returned by the API
   * @return the users authentication token
   * @throws ClassNotFoundException if the {@code UserProfile} class cannot be
   *     located
   * @throws InvocationTargetException if an error occurs while invoking
   *     {@code getToken} on the given object
   * @throws NoSuchMethodException if the {@code getToken} method cannot be
   *     found.
   * @throws IllegalAccessException if the {@code getToken} method cannot be
   *     accessed.
   */
  @VisibleForTesting
  String retrieveToken(Object userProfile) throws ClassNotFoundException,
      InvocationTargetException, NoSuchMethodException, IllegalAccessException {
    Class<?> paramTypes[] = new Class<?>[] {};
    Object argList[] = new Object[] {};
    Method getToken = userProfile.getClass().getMethod("getToken", paramTypes);
    return (String) getToken.invoke(userProfile, argList);
  }

  /**
   * Generates a new token by impersonating a network.
   *
   * @param stubPackage the package where all generated stubs are located
   * @param session the DFA service session
   * @return an impersonated token
   * @throws LoginTicketException thrown if generating a token fails
   */
  public String generateImpersonateNetworkToken(String stubPackage, DfaServiceSession session)
      throws LoginTicketException {
    try {
      Class<?> loginClass = getLoginClass(stubPackage);
      Object loginService = getLoginService(session, loginClass);

      Object userProfile = impersonateNetwork(session, loginClass, loginService);

      return retrieveToken(userProfile);
    } catch (ClassNotFoundException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    } catch (NoSuchMethodException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    } catch (InvocationTargetException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e.getCause());
    } catch (IllegalAccessException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    }
  }

  /**
   * Impersonates a network using the DFA API.
   *
   * @param loginClass the {@code Class} type of the login service
   * @param loginService a service client object wrapping the login service
   * @return the {@code UserProfile} response from the API
   * @throws NoSuchMethodException
   * @throws SecurityException
   * @throws IllegalArgumentException if one of the arguments used in
   *     authenticating is not allowed
   * @throws IllegalAccessException if the {@code authenticate} method cannot be
   *     accessed
   * @throws InvocationTargetException if an error occurs while invoking the
   *     {@code authenticate} method
   * @throws NoSuchMethodException if the {@code authenticate method} cannot be
   *     located
   */
  @VisibleForTesting
  Object impersonateNetwork(DfaServiceSession session, Class<?> loginClass, Object loginService)
      throws SecurityException, NoSuchMethodException, IllegalArgumentException,
      IllegalAccessException, InvocationTargetException {
    Class<?> paramTypes[] = {String.class, String.class, long.class};
    Method authMethod = loginClass.getDeclaredMethod("impersonateNetwork", paramTypes);
    Object argList[] = {session.getAdsUser().getUsername(), session.getToken(),
        session.getNetworkToImpersonate()};
    return authMethod.invoke(loginService, argList);
  }

  /**
   * Generates a new token by impersonating a user.
   *
   * @param stubPackage the package where all generated stubs are located
   * @param session the DFA service session
   * @return an impersonated token
   * @throws LoginTicketException thrown if generating a token fails
   */
  public String generateImpersonateUserToken(String stubPackage, DfaServiceSession session)
      throws LoginTicketException {
    try {
      Class<?> loginClass = getLoginClass(stubPackage);
      Object loginService = getLoginService(session, loginClass);

      Object userProfile = impersonateUser(session, loginClass, loginService);

      return retrieveToken(userProfile);
    } catch (ClassNotFoundException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    } catch (NoSuchMethodException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    } catch (InvocationTargetException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e.getCause());
    } catch (IllegalAccessException e) {
      throw new LoginTicketException(
          "Unable to generate authentication token because of error:", e);
    }
  }

  /**
   * Impersonates a user using the DFA API.
   *
   * @param loginClass the {@code Class} type of the login service
   * @param loginService a service client object wrapping the login service
   * @return the {@code UserProfile} response from the API
   * @throws NoSuchMethodException
   * @throws SecurityException
   * @throws IllegalArgumentException if one of the arguments used in
   *     authenticating is not allowed
   * @throws IllegalAccessException if the {@code authenticate} method cannot be
   *     accessed
   * @throws InvocationTargetException if an error occurs while invoking the
   *     {@code authenticate} method
   * @throws NoSuchMethodException if the {@code authenticate method} cannot be
   *     located
   */
  @VisibleForTesting
  Object impersonateUser(DfaServiceSession session, Class<?> loginClass, Object loginService)
      throws SecurityException, NoSuchMethodException, IllegalArgumentException,
      IllegalAccessException, InvocationTargetException {
    Class<?> paramTypes[] = {String.class, String.class, String.class};
    Method authMethod = loginClass.getDeclaredMethod("impersonateUser", paramTypes);
    Object argList[] = {session.getAdsUser().getUsername(), session.getToken(),
        session.getUsernameToImpersonate()};
    return authMethod.invoke(loginService, argList);
  }
}
