// Copyright 2011 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.api.ads.dfa.lib.client;

import com.google.api.ads.common.lib.client.AdsServiceSession;

/**
 * A {@code DfaServiceSession} represents a single session of DFA use.
 *
 * @author api.arogal@gmail.com (Adam Rogal)
 */
public class DfaServiceSession extends AdsServiceSession<DfaUser> {
  private String token;

  private String usernameToImpersonate;
  private Long networkToImpersonate;
  private String impersonatedToken;

  /**
   * Constructor.
   *
   * @param dfaUser the user of this session
   */
  public DfaServiceSession(DfaUser dfaUser) {
    super(dfaUser);
  }

  /**
   * Sets the username to impersonate. Only superusers may impersonate.
   *
   * Only a username to impersonate or a network to impersonate should ever be
   * specified at the same time.
   */
  public void setUsernameToImpersonate(String usernameToImpersonate) {
    this.usernameToImpersonate = usernameToImpersonate;
  }

  /**
   * Gets the username to impersonate. Only superusers may impersonate.
   */
  public String getUsernameToImpersonate() {
    return usernameToImpersonate;
  }

  /**
   * Sets the network to impersonate. Only superusers may impersonate.
   *
   * Only a username to impersonate or a network to impersonate should ever be
   * specified at the same time.
   */
  public void setNetworkToImpersonate(Long networkToImpersonate) {
    this.networkToImpersonate = networkToImpersonate;
  }

  /**
   * Gets the network to impersonate. Only superusers may impersonate.
   */
  public Long getNetworkToImpersonate() {
    return networkToImpersonate;
  }

  /**
   * Sets the token necessary to impersonate. Only superusers may impersonate.
   */
  public void setImpersonatedToken(String impersonatedToken) {
    this.impersonatedToken = impersonatedToken;
  }

  /**
   * Gets the token necessary to impersonate. Only superusers may impersonate.
   */
  public String getImpersonatedToken() {
    return impersonatedToken;
  }

  /**
   * Indicates whether clients of this service session will impersonate. A
   * username or network ID is required to impersonate; just an impersonated
   * token is not enough.
   */
  public boolean isImpersonating() {
    return (usernameToImpersonate != null || networkToImpersonate != null);
  }

  /**
   * Resets all settings related to impersonating in this service session.
   *
   * You should call this method when switching between impersonating one
   * username/network and another to ensure that the old impersonation token
   * has been cleared.
   */
  public void stopImpersonating() {
    usernameToImpersonate = impersonatedToken = null;
    networkToImpersonate = null;
  }

  /**
   * Gets the token associated with this session.
   */
  public String getToken() {
    return token;
  }

  /**
   * Sets the token associated with this session.
   */
  public void setToken(String token) {
    this.token = token;
  }
}
