// Copyright 2011 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.api.ads.dfa.lib.client;

import com.google.api.ads.common.lib.client.AdsUser;
import com.google.api.ads.common.lib.exception.ValidationException;

import java.util.Properties;

/**
 * Immutable class representing a DFA user.
 *
 * @author api.arogal@gmail.com (Adam Rogal)
 * @author api.jdilallo@gmail.com (Joseph DiLallo)
 */
public class DfaUser extends AdsUser {

  private String username;
  private String password;
  private String authenticationToken;
  private String applicationName;

  /**
   * Default constructor.
   */
  private DfaUser() {
    super();
  }

  /**
   * Gets the user name.
   */
  public String getUsername() {
    return username;
  }

  /**
   * Gets the password.
   */
  public String getPassword() {
    return password;
  }

  /**
   * Gets the token used for authentication.
   */
  public String getToken() {
    return authenticationToken;
  }

  /**
   * Gets the application name.
   */
  public String getApplicationName() {
    return applicationName;
  }

  /**
   * DFA user builder.
   */
  public static class Builder extends AdsUser.Builder {

    private static final String DEFAULT_PROPERTIES_PATH = "/dfa.properties";
    private DfaUser dfaUser;

    /**
     * Constructor.
     */
    public Builder() {
      dfaUser = new DfaUser();
    }

    /**
     * Loads credentials from the default resource location.
     */
    public Builder fromProperties() {
      return fromProperties(DEFAULT_PROPERTIES_PATH);
    }

    /**
     * Loads credentials from a specified path. The path will first be
     * treated as a resource. If no resource can be found with that path, it
     * will be tried as a file path.
     */
    public Builder fromProperties(String path) {
      Properties credentials = getProperties(path);

      dfaUser.username = credentials.getProperty("api.dfa.username", null);
      dfaUser.password = credentials.getProperty("api.dfa.password", null);
      dfaUser.applicationName = credentials.getProperty("api.dfa.applicationName", null);
      dfaUser.authenticationToken = credentials.getProperty("api.dfa.token", null);

      return this;
    }

    /**
     * Includes username and password.
     */
    public Builder withUsernameAndPassword(String username, String password) {
      dfaUser.username = username;
      dfaUser.password = password;
      return this;
    }

    /**
     * Includes a username with a hard-coded token to be used instead of
     * fetching a new one.
     */
    public Builder withUsernameAndToken(String username, String token) {
      dfaUser.username = username;
      dfaUser.authenticationToken = token;
      return this;
    }

    /**
     * Includes application name.
     */
    public Builder withApplicationName(String applicationName) {
      dfaUser.applicationName = applicationName;
      return this;
    }

    /**
     * Builds the {@code DfaUser}.
     * @return the built {@code DfaUser}
     * @throws ValidationException if the {@code DfaUser} did not validate
     */
    public DfaUser build() throws ValidationException {
      validate(dfaUser);
      return dfaUser;
    }

    /**
     * Validates the DFA user.
     */
    private void validate(DfaUser dfaUser) throws ValidationException {
      // Check for valid username.
      if (dfaUser.username == null) {
        throw new ValidationException("Username is required.", "username");
      }

      boolean usingPassword = dfaUser.password != null;
      boolean usingToken = dfaUser.authenticationToken != null;

      // Check that at least one auth mechanism is being use.
      if (!usingPassword && !usingToken) {
        throw new ValidationException(
            "Either username/password or username/token must be used.",
            "");
      }

      // Check that only one is being used.
      if (usingPassword && usingToken) {
        throw new ValidationException(
            "Both username/password and username/token cannot be used at the same time.",
            "");
      }
    }
  }
}
