// Copyright 2011 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.api.ads.dfa.lib.factory;

import com.google.api.ads.dfa.lib.client.DfaServiceSession;

/**
 * Utility class to create DFA service clients. The session is stored
 * as a reference within the client so that the session may be altered to
 * affect the behavior of the client.
 * <p>
 * Usage example:
 * <pre><code>
 * DfaUser user = new DfaUser.Builder()...build();
 * DfaServiceSession session = new DfaServiceSession(user);
 * CampaignRemote campaignService =
 *     DfaServices.get(session, CampaignRemote.class);
 * campaignService.makeRequest(...);
 * ...
 * session.setToken(...);
 * campaignService.makeRequest(...);
 * </code></pre>
 *
 * @author api.jdilallo@gmail.com (Joseph DiLallo)
 */
public final class DfaServices {

  private static final DfaServiceClientFactory factory = new DfaServiceClientFactory();

  /**
   * Private constructor.
   */
  private DfaServices() {}

  /**
   * Gets a client for the service represented by the interface with a reference
   * to the session.
   *
   * @param <T> the service type
   * @param dfaServiceSession the DFA service session
   * @param interfaceClass the service interface class
   * @return the client for the service
   */
  public static <T> T get(DfaServiceSession dfaServiceSession,
      Class<T> interfaceClass) {
    return factory.getServiceClientAsInterface(dfaServiceSession, interfaceClass);
  }

  /**
   * Gets a client for the service represented by the interface with a reference
   * to the session.
   *
   * @param <T> the service type
   * @param dfaServiceSession the DFA service session
   * @param interfaceClass the service interface class
   * @param endpointServer the endpoint server to make requests to
   * @return the client for the service
   */
  public static <T> T get(DfaServiceSession dfaServiceSession,
      Class<T> interfaceClass, String endpointServer) {
    return factory.getServiceClientAsInterface(dfaServiceSession, interfaceClass,
        endpointServer);
  }
}
